package thesis.gui;

import java.awt.FileDialog;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.GroupLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.SpinnerNumberModel;
import javax.swing.GroupLayout.Alignment;
import javax.swing.LayoutStyle.ComponentPlacement;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import thesis.simulation.UniverseModel;

public final class ModelDialog extends JDialog {
  
  private static final long serialVersionUID = 1L;

  private List<Galaxy> galaxies;
  
  private Universe universe;
  private JButton deleteButton;
  private JCheckBox openSimulator;
  
  private SpinnerNumberModel galaxyIndex;
  private JSpinner positionXSpinner;
  private JSpinner positionYSpinner;
  private JSpinner velocityXSpinner;
  private JSpinner velocityYSpinner;
  private JSpinner radiusSpinner;
  private JSpinner massSpinner;
  private JSpinner numStarsSpinner;
  private JComboBox distributionComboBox;
  
  public ModelDialog(Window owner) {
    super(owner, "Model", DEFAULT_MODALITY_TYPE);
    setDefaultCloseOperation(DISPOSE_ON_CLOSE);
    
    galaxies = new ArrayList<Galaxy>();

    setResizable(false);
    setContentPane(createContentPanel());
    pack();
    setLocationRelativeTo(owner);
  }
  
  private void setGalaxyModels() {
    Galaxy galaxy = galaxies.get(galaxyIndex.getNumber().intValue() - 1);
    positionXSpinner.setModel(galaxy.getPositionX());
    positionYSpinner.setModel(galaxy.getPositionY());
    velocityXSpinner.setModel(galaxy.getVelocityX());
    velocityYSpinner.setModel(galaxy.getVelocityY());
    radiusSpinner.setModel(galaxy.getRadius());
    massSpinner.setModel(galaxy.getMass());
    numStarsSpinner.setModel(galaxy.getNumberOfStars());
    distributionComboBox.setModel(galaxy.getDistibution());
  }
 
  private void addGalaxy(Galaxy galaxy) {
    galaxy.getPositionX().addChangeListener(universe);
    galaxy.getPositionY().addChangeListener(universe);
    galaxy.getVelocityX().addChangeListener(universe);
    galaxy.getVelocityY().addChangeListener(universe);
    galaxy.getRadius().addChangeListener(universe);
    
    galaxies.add(galaxy);
    galaxyIndex.setMinimum(1);
    galaxyIndex.setMaximum(galaxies.size());
    galaxyIndex.setValue(galaxies.size());
    if (galaxies.size() > 1) {
      deleteButton.setEnabled(true);
    }
    
    universe.repaint();
  }
  
  private void deleteCurrentGalaxy() {
    int index = galaxyIndex.getNumber().intValue() - 1;
    galaxies.remove(index);
    if (index == galaxies.size()) {
      galaxyIndex.setValue(index);
    }
    galaxyIndex.setMaximum(galaxies.size());
    if (galaxies.size() == 1) {
      deleteButton.setEnabled(false);
    }
    
    universe.repaint();
  }
  
  private void generateUniverse(String filename) {
    try {
      UniverseModel.save(galaxies, filename);
      if (openSimulator.isSelected()) {
        dispose();
        new SimulationDialog(getOwner(), filename).setVisible(true);
      } else {
        Application.showMessage("Model je uspješno zapisan.");
      }
    } catch (FileNotFoundException e) {
      Application.showError(
          "Odabranu datoteku nije moguće koristiti za pisanje.");
    } catch (IOException e) {
      Application.showError("Dogodila se greška tijekom zapisivanje modela.");
    }
  }

  private JPanel createContentPanel() {
    galaxyIndex = new SpinnerNumberModel(1, 1, 1, 1);
    JLabel galaxyIndexLabel = new JLabel("Galaksija:");
    JSpinner galaxyIndexSpinner = new JSpinner(galaxyIndex);
    galaxyIndexSpinner.addChangeListener(new ChangeListener() {
      @Override
      public void stateChanged(ChangeEvent e) {
        setGalaxyModels();
      }
    });

    deleteButton = new JButton("Izbriši");
    deleteButton.setEnabled(false);
    deleteButton.addActionListener(new ActionListener() {
      @Override
      public void actionPerformed(ActionEvent e) {
        deleteCurrentGalaxy();
      }
    });
    
    JButton addButton = new JButton("Stvori novu");
    addButton.addActionListener(new ActionListener() {
      @Override
      public void actionPerformed(ActionEvent e) {
        addGalaxy(new Galaxy());
      }
    });
    
    universe = new Universe(galaxies, galaxyIndex);

    JPanel optionsPanel = createOptionsPanel();
    
    JLabel fileLabel = new JLabel("Datoteka:");
    final JTextField fileTextField = new JTextField();
    
    JButton fileButton = new JButton("...");
    fileButton.addActionListener(new ActionListener() {
      @Override
      public void actionPerformed(ActionEvent e) {
        Application.openFileDialog(FileDialog.SAVE, fileTextField);
      }
    });    
    
    JButton generateButton = new JButton("Stvori model");
    openSimulator = new JCheckBox("i otvori simulator");
    
    generateButton.addActionListener(new ActionListener() {
      @Override
      public void actionPerformed(ActionEvent e) {
        generateUniverse(fileTextField.getText());
      }
    });
    
    JPanel panel = new JPanel();
    GroupLayout layout = new GroupLayout(panel);
    panel.setLayout(layout);
    
    layout.setHorizontalGroup(layout.createSequentialGroup()
      .addContainerGap()
      .addGroup(layout.createParallelGroup()
        .addGroup(layout.createSequentialGroup()
          .addComponent(galaxyIndexLabel)
          .addPreferredGap(ComponentPlacement.RELATED)
          .addComponent(galaxyIndexSpinner, 60, 60, 60)
          .addPreferredGap(ComponentPlacement.UNRELATED)
          .addComponent(deleteButton)
          .addPreferredGap(ComponentPlacement.RELATED)
          .addComponent(addButton)
        )
        .addComponent(optionsPanel)
        .addComponent(universe)
        .addGroup(layout.createSequentialGroup()
          .addComponent(fileLabel)
          .addPreferredGap(ComponentPlacement.RELATED)
          .addComponent(fileTextField)
          .addPreferredGap(ComponentPlacement.RELATED)
          .addComponent(fileButton)
        )
        .addGroup(layout.createSequentialGroup()
          .addComponent(generateButton)
          .addPreferredGap(ComponentPlacement.RELATED)
          .addComponent(openSimulator)
        )
      )
      .addContainerGap()
    );
    
    layout.setVerticalGroup(layout.createSequentialGroup()
      .addContainerGap()
      .addGroup(layout.createParallelGroup(Alignment.BASELINE)
        .addComponent(galaxyIndexLabel)
        .addComponent(galaxyIndexSpinner)
        .addComponent(deleteButton)
        .addComponent(addButton)
      )
      .addPreferredGap(ComponentPlacement.UNRELATED)
      .addComponent(optionsPanel)
      .addPreferredGap(ComponentPlacement.UNRELATED)
      .addComponent(universe)
      .addPreferredGap(ComponentPlacement.UNRELATED)
      .addGroup(layout.createParallelGroup(Alignment.BASELINE)
        .addComponent(fileLabel)
        .addComponent(fileTextField)
        .addComponent(fileButton)
      )
      .addPreferredGap(ComponentPlacement.RELATED)
        .addGroup(layout.createParallelGroup(Alignment.BASELINE)
          .addComponent(generateButton)
          .addComponent(openSimulator)
        )
      .addContainerGap()
    );
    
    return panel;
  }

  private JPanel createOptionsPanel() {
    Galaxy galaxy = new Galaxy();
    
    JLabel positionLabel = new JLabel("Položaj:");
    positionXSpinner = new JSpinner(galaxy.getPositionX());
    positionYSpinner = new JSpinner(galaxy.getPositionY());

    JLabel velocityLabel = new JLabel("Brzina:");
    velocityXSpinner = new JSpinner(galaxy.getVelocityX());
    velocityYSpinner = new JSpinner(galaxy.getVelocityY());

    JLabel radiusLabel = new JLabel("Polumjer:");
    radiusSpinner = new JSpinner(galaxy.getRadius());
    
    JLabel massLabel = new JLabel("Masa:");
    massSpinner = new JSpinner(galaxy.getMass());
    
    JLabel numStarsLabel = new JLabel("Broj zvijezda:");
    numStarsSpinner = new JSpinner(galaxy.getNumberOfStars());
    
    JLabel distributionLabel = new JLabel("Raspodjela:");
    distributionComboBox = new JComboBox(galaxy.getDistibution());
    
    addGalaxy(galaxy);
    
    JPanel panel = new JPanel();
    GroupLayout layout = new GroupLayout(panel);
    panel.setLayout(layout);
    panel.setBorder(BorderFactory.createEtchedBorder());
    
    layout.setHorizontalGroup(layout.createSequentialGroup()
      .addContainerGap(GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
      .addGroup(layout.createParallelGroup(Alignment.TRAILING)
        .addComponent(positionLabel)
        .addComponent(velocityLabel)
      )
      .addPreferredGap(ComponentPlacement.RELATED)
      .addGroup(layout.createParallelGroup()
        .addComponent(positionXSpinner, 70, 70, GroupLayout.DEFAULT_SIZE)
        .addComponent(velocityXSpinner, 70, 70, GroupLayout.DEFAULT_SIZE)
      )
      .addPreferredGap(ComponentPlacement.RELATED)
      .addGroup(layout.createParallelGroup()
        .addComponent(positionYSpinner, 70, 70, GroupLayout.DEFAULT_SIZE)
        .addComponent(velocityYSpinner, 70, 70, GroupLayout.DEFAULT_SIZE)
      )
      .addPreferredGap(ComponentPlacement.UNRELATED)
      .addGroup(layout.createParallelGroup(Alignment.TRAILING)
        .addComponent(radiusLabel)
        .addComponent(massLabel)
      )
      .addPreferredGap(ComponentPlacement.RELATED)
      .addGroup(layout.createParallelGroup()
        .addComponent(radiusSpinner, 70, 70, GroupLayout.DEFAULT_SIZE)
        .addComponent(massSpinner, 70, 70, GroupLayout.DEFAULT_SIZE)
      )
      .addPreferredGap(ComponentPlacement.UNRELATED)
      .addGroup(layout.createParallelGroup(Alignment.TRAILING)
        .addComponent(numStarsLabel)
        .addComponent(distributionLabel)
      )
      .addPreferredGap(ComponentPlacement.RELATED)
      .addGroup(layout.createParallelGroup()
        .addComponent(numStarsSpinner, 110, 110, GroupLayout.DEFAULT_SIZE)
        .addComponent(distributionComboBox, 110, 110, GroupLayout.DEFAULT_SIZE)
      )
      .addContainerGap(GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
    );
    
    layout.setVerticalGroup(layout.createSequentialGroup()
      .addContainerGap()
      .addGroup(layout.createParallelGroup(Alignment.BASELINE)
        .addComponent(positionLabel)
        .addComponent(positionXSpinner)
        .addComponent(positionYSpinner)
        .addComponent(radiusLabel)
        .addComponent(radiusSpinner)
        .addComponent(numStarsLabel)
        .addComponent(numStarsSpinner)
      )
      .addPreferredGap(ComponentPlacement.UNRELATED)
      .addGroup(layout.createParallelGroup(Alignment.BASELINE)
        .addComponent(velocityLabel)
        .addComponent(velocityXSpinner)
        .addComponent(velocityYSpinner)
        .addComponent(massLabel)
        .addComponent(massSpinner)
        .addComponent(distributionLabel)
        .addComponent(distributionComboBox)
      )
      .addContainerGap()
    );
    
    return panel;
  }

}
